@@ -6,54 +6,55 @@
 #import "XADCRCHandle.h"
 #import "XADPlatform.h"
 
-#import "XADZipParser.h"
-#import "XADZipSFXParsers.h"
-#import "XADRARParser.h"
 #import "XAD7ZipParser.h"
+#import "XADALZipParser.h"
+#import "XADAppleSingleParser.h"
+#import "XADARCParser.h"
+#import "XADARJParser.h"
+#import "XADArParser.h"
+#import "XADBinHexParser.h"
+#import "XADBzip2Parser.h"
+#import "XADCABParser.h"
+#import "XADCFBFParser.h"
+#import "XADCompactProParser.h"
+#import "XADCompressParser.h"
+#import "XADCpioParser.h"
+#import "XADCrunchParser.h"
+#import "XADDiskDoublerParser.h"
+#import "XADGzipParser.h"
+#import "XADISO9660Parser.h"
+#import "XADLBRParser.h"
+#import "XADLibXADParser.h"
+#import "XADLZHParser.h"
+#import "XADLZHSFXParsers.h"
+#import "XADLZMAAloneParser.h"
+#import "XADLZXParser.h"
+#import "XADMacBinaryParser.h"
+#import "XADNDSParser.h"
+#import "XADNowCompressParser.h"
+#import "XADNSAParser.h"
+#import "XADNSISParser.h"
+#import "XADPackItParser.h"
+#import "XADPDFParser.h"
+#import "XADPowerPackerParser.h"
+#import "XADPPMdParser.h"
+#import "XADRARParser.h"
+#import "XADRPMParser.h"
+#import "XADSARParser.h"
+#import "XADSplitFileParser.h"
+#import "XADSqueezeParser.h"
 #import "XADStuffItParser.h"
 #import "XADStuffIt5Parser.h"
 #import "XADStuffItSplitParser.h"
 #import "XADStuffItXParser.h"
-#import "XADCompactProParser.h"
-#import "XADDiskDoublerParser.h"
-#import "XADBinHexParser.h"
-#import "XADMacBinaryParser.h"
-#import "XADAppleSingleParser.h"
-#import "XADPackItParser.h"
-#import "XADNowCompressParser.h"
-#import "XADGzipParser.h"
-#import "XADBzip2Parser.h"
-#import "XADLZMAAloneParser.h"
-#import "XADXZParser.h"
-#import "XADCompressParser.h"
-#import "XADTarParser.h"
-#import "XADCpioParser.h"
-#import "XADXARParser.h"
-#import "XADArParser.h"
-#import "XADRPMParser.h"
-#import "XADLZXParser.h"
-#import "XADPowerPackerParser.h"
-#import "XADLZHParser.h"
-#import "XADLZHSFXParsers.h"
-#import "XADARJParser.h"
-#import "XADARCParser.h"
-#import "XADSqueezeParser.h"
-#import "XADCrunchParser.h"
-#import "XADLBRParser.h"
-#import "XADZooParser.h"
-#import "XADNSISParser.h"
-#import "XADCABParser.h"
-#import "XADPPMdParser.h"
 #import "XADSWFParser.h"
-#import "XADPDFParser.h"
-#import "XADALZipParser.h"
-#import "XADCFBFParser.h"
-#import "XADNDSParser.h"
-#import "XADNSAParser.h"
-#import "XADSARParser.h"
-#import "XADSplitFileParser.h"
-#import "XADISO9660Parser.h"
-#import "XADLibXADParser.h"
+#import "XADTarParser.h"
+#import "XADWARCParser.h"
+#import "XADXARParser.h"
+#import "XADXZParser.h"
+#import "XADZipParser.h"
+#import "XADZipSFXParsers.h"
+#import "XADZooParser.h"
 
 #include <dirent.h>
 
@@ -175,6 +176,7 @@ static int maxheader=0;
 		[XADNSAParser class],
 		[XADSARParser class],
 		[XADArParser class],
+		[XADWARCParser class],
 
 		// Detectors that require lots of work
 		[XADWinZipSFXParser class],
@@ -213,7 +215,7 @@ static int maxheader=0;
 }
 
 +(Class)archiveParserClassForHandle:(CSHandle *)handle firstBytes:(NSData *)header
-name:(NSString *)name propertiesToAdd:(NSMutableDictionary *)props
+resourceFork:(XADResourceFork *)fork name:(NSString *)name propertiesToAdd:(NSMutableDictionary *)props
 {
 	NSEnumerator *enumerator=[parserclasses objectEnumerator];
 	Class parserclass;
@@ -224,7 +226,7 @@ name:(NSString *)name propertiesToAdd:(NSMutableDictionary *)props
 
 		@try {
 			if([parserclass recognizeFileWithHandle:handle firstBytes:header
-			name:name propertiesToAdd:props])
+			resourceFork:fork name:name propertiesToAdd:props])
 			{
 				[handle seekToFileOffset:0];
 				return parserclass;
@@ -236,37 +238,65 @@ name:(NSString *)name propertiesToAdd:(NSMutableDictionary *)props
 
 +(XADArchiveParser *)archiveParserForHandle:(CSHandle *)handle name:(NSString *)name
 {
-	NSData *header=[handle readDataOfLengthAtMost:maxheader];
-	return [self archiveParserForHandle:handle firstBytes:header name:name];
+	return [self archiveParserForHandle:handle resourceFork:nil name:name];
 }
 
 +(XADArchiveParser *)archiveParserForHandle:(CSHandle *)handle name:(NSString *)name error:(XADError *)errorptr
 {
 	if(errorptr) *errorptr=XADNoError;
-	@try { return [self archiveParserForHandle:handle name:name]; }
+	@try { return [self archiveParserForHandle:handle resourceFork:nil name:name]; }
+	@catch(id exception) { if(errorptr) *errorptr=[XADException parseException:exception]; }
+	return nil;
+}
+
++(XADArchiveParser *)archiveParserForHandle:(CSHandle *)handle resourceFork:(XADResourceFork *)fork name:(NSString *)name
+{
+	NSData *header=[handle readDataOfLengthAtMost:maxheader];
+	return [self archiveParserForHandle:handle firstBytes:header resourceFork:fork name:name];
+}
+
++(XADArchiveParser *)archiveParserForHandle:(CSHandle *)handle resourceFork:(XADResourceFork *)fork name:(NSString *)name error:(XADError *)errorptr
+{
+	if(errorptr) *errorptr=XADNoError;
+	@try { return [self archiveParserForHandle:handle resourceFork:fork name:name]; }
 	@catch(id exception) { if(errorptr) *errorptr=[XADException parseException:exception]; }
 	return nil;
 }
 
 +(XADArchiveParser *)archiveParserForHandle:(CSHandle *)handle firstBytes:(NSData *)header name:(NSString *)name
+{
+	return [self archiveParserForHandle:handle firstBytes:header resourceFork:nil name:name];
+}
+
++(XADArchiveParser *)archiveParserForHandle:(CSHandle *)handle firstBytes:(NSData *)header name:(NSString *)name error:(XADError *)errorptr
+{
+	if(errorptr) *errorptr=XADNoError;
+	@try { return [self archiveParserForHandle:handle firstBytes:header resourceFork:nil name:name]; }
+	@catch(id exception) { if(errorptr) *errorptr=[XADException parseException:exception]; }
+	return nil;
+}
+
++(XADArchiveParser *)archiveParserForHandle:(CSHandle *)handle firstBytes:(NSData *)header resourceFork:(XADResourceFork *)fork name:(NSString *)name;
 {
 	NSMutableDictionary *props=[NSMutableDictionary dictionary];
 
-	Class parserclass=[self archiveParserClassForHandle:handle
-	firstBytes:header name:name propertiesToAdd:props];
+	Class parserclass=[self archiveParserClassForHandle:handle firstBytes:header
+	resourceFork:fork name:name propertiesToAdd:props];
 
-	XADArchiveParser *parser=[[[parserclass alloc] initWithHandle:handle
-	name:name] autorelease];
+	XADArchiveParser *parser=[[parserclass new] autorelease];
+	[parser setHandle:handle];
+	[parser setResourceFork:fork];
+	[parser setName:name];
 
 	[parser addPropertiesFromDictionary:props];
 
 	return parser;
 }
 
-+(XADArchiveParser *)archiveParserForHandle:(CSHandle *)handle firstBytes:(NSData *)header name:(NSString *)name error:(XADError *)errorptr
++(XADArchiveParser *)archiveParserForHandle:(CSHandle *)handle firstBytes:(NSData *)header resourceFork:(XADResourceFork *)fork name:(NSString *)name error:(XADError *)errorptr
 {
 	if(errorptr) *errorptr=XADNoError;
-	@try { return [self archiveParserForHandle:handle firstBytes:header name:name]; }
+	@try { return [self archiveParserForHandle:handle firstBytes:header resourceFork:fork name:name]; }
 	@catch(id exception) { if(errorptr) *errorptr=[XADException parseException:exception]; }
 	return nil;
 }
@@ -274,12 +304,15 @@ name:(NSString *)name propertiesToAdd:(NSMutableDictionary *)props
 +(XADArchiveParser *)archiveParserForPath:(NSString *)filename
 {
 	CSHandle *handle=[CSFileHandle fileHandleForReadingAtPath:filename];
-
 	NSData *header=[handle readDataOfLengthAtMost:maxheader];
+
+	CSHandle *forkhandle=[XADPlatform handleForReadingResourceForkAtPath:filename];
+	XADResourceFork *fork=[XADResourceFork resourceForkWithHandle:forkhandle error:NULL];
+
 	NSMutableDictionary *props=[NSMutableDictionary dictionary];
 
 	Class parserclass=[self archiveParserClassForHandle:handle
-	firstBytes:header name:filename propertiesToAdd:props];
+	firstBytes:header resourceFork:fork name:filename propertiesToAdd:props];
 	if(!parserclass) return nil;
 
 	// Attempt to create a multi-volume parser, if we can find the volumes.
@@ -301,10 +334,10 @@ name:(NSString *)name propertiesToAdd:(NSMutableDictionary *)props
 
 				CSMultiHandle *multihandle=[CSMultiHandle multiHandleWithHandleArray:handles];
 
-				XADArchiveParser *parser=[[[parserclass alloc] initWithHandle:multihandle
-				name:filename] autorelease];
-
-				[props setObject:volumes forKey:XADVolumesKey];
+				XADArchiveParser *parser=[[parserclass new] autorelease];
+				[parser setHandle:multihandle];
+				[parser setResourceFork:fork];
+				[parser setAllFilenames:volumes];
 				[parser addPropertiesFromDictionary:props];
 
 				return parser;
@@ -319,8 +352,11 @@ name:(NSString *)name propertiesToAdd:(NSMutableDictionary *)props
 	}
 	@catch(id e) { } // Fall through to a single file instead.
 
-	XADArchiveParser *parser=[[[parserclass alloc] initWithHandle:handle
-	name:filename] autorelease];
+	XADArchiveParser *parser=[[parserclass new] autorelease];
+	[parser setHandle:handle];
+	[parser setResourceFork:fork];
+	[parser setFilename:filename];
+	[parser addPropertiesFromDictionary:props];
 
 	[props setObject:[NSArray arrayWithObject:filename] forKey:XADVolumesKey];
 	[parser addPropertiesFromDictionary:props];
@@ -338,11 +374,39 @@ name:(NSString *)name propertiesToAdd:(NSMutableDictionary *)props
 
 +(XADArchiveParser *)archiveParserForEntryWithDictionary:(NSDictionary *)entry archiveParser:(XADArchiveParser *)parser wantChecksum:(BOOL)checksum
 {
+	return [self archiveParserForEntryWithDictionary:entry resourceForkDictionary:nil archiveParser:parser wantChecksum:checksum];
+}
+
++(XADArchiveParser *)archiveParserForEntryWithDictionary:(NSDictionary *)entry archiveParser:(XADArchiveParser *)parser wantChecksum:(BOOL)checksum error:(XADError *)errorptr
+{
+	if(errorptr) *errorptr=XADNoError;
+	@try { return [self archiveParserForEntryWithDictionary:entry resourceForkDictionary:nil archiveParser:parser wantChecksum:checksum]; }
+	@catch(id exception) { if(errorptr) *errorptr=[XADException parseException:exception]; }
+	return nil;
+}
+
++(XADArchiveParser *)archiveParserForEntryWithDictionary:(NSDictionary *)entry resourceForkDictionary:(NSDictionary *)forkentry archiveParser:(XADArchiveParser *)parser wantChecksum:(BOOL)checksum
+{
+	XADResourceFork *fork=nil;
+	if(forkentry)
+	{
+		CSHandle *forkhandle=[parser handleForEntryWithDictionary:forkentry wantChecksum:checksum];
+		if(forkhandle)
+		{
+			fork=[XADResourceFork resourceForkWithHandle:forkhandle];
+			if(checksum && [forkhandle hasChecksum])
+			{
+				[forkhandle seekToEndOfFile];
+				if(![forkhandle isChecksumCorrect]) [XADException raiseChecksumException];
+			}
+		}
+	}
+
 	CSHandle *handle=[parser handleForEntryWithDictionary:entry wantChecksum:checksum];
 	if(!handle) [XADException raiseNotSupportedException];
 
 	NSString *filename=[[entry objectForKey:XADFileNameKey] string];
-	XADArchiveParser *subparser=[XADArchiveParser archiveParserForHandle:handle name:filename];
+	XADArchiveParser *subparser=[XADArchiveParser archiveParserForHandle:handle resourceFork:fork name:filename];
 	if(!subparser) return nil;
 
 	if([parser hasPassword]) [subparser setPassword:[parser password]];
@@ -352,10 +416,10 @@ name:(NSString *)name propertiesToAdd:(NSMutableDictionary *)props
 	return subparser;
 }
 
-+(XADArchiveParser *)archiveParserForEntryWithDictionary:(NSDictionary *)entry archiveParser:(XADArchiveParser *)parser wantChecksum:(BOOL)checksum error:(XADError *)errorptr
++(XADArchiveParser *)archiveParserForEntryWithDictionary:(NSDictionary *)entry resourceForkDictionary:(NSDictionary *)forkentry archiveParser:(XADArchiveParser *)parser wantChecksum:(BOOL)checksum error:(XADError *)errorptr
 {
 	if(errorptr) *errorptr=XADNoError;
-	@try { return [self archiveParserForEntryWithDictionary:entry archiveParser:parser wantChecksum:checksum]; }
+	@try { return [self archiveParserForEntryWithDictionary:entry resourceForkDictionary:forkentry archiveParser:parser wantChecksum:checksum]; }
 	@catch(id exception) { if(errorptr) *errorptr=[XADException parseException:exception]; }
 	return nil;
 }
@@ -364,13 +428,13 @@ name:(NSString *)name propertiesToAdd:(NSMutableDictionary *)props
 
 
 
--(id)initWithHandle:(CSHandle *)handle name:(NSString *)name
+-(id)init
 {
 	if((self=[super init]))
 	{
-		sourcehandle=[handle retain];
-
+		sourcehandle=nil;
 		skiphandle=nil;
+		resourcefork=nil;
 		delegate=nil;
 		password=nil;
 		passwordencodingname=nil;
@@ -378,9 +442,7 @@ name:(NSString *)name propertiesToAdd:(NSMutableDictionary *)props
 
 		stringsource=[XADStringSource new];
 
-		properties=[[NSMutableDictionary alloc] initWithObjectsAndKeys:
-			[name lastPathComponent],XADArchiveNameKey,
-		nil];
+		properties=[NSMutableDictionary new];
 
 		currsolidobj=nil;
 		currsolidhandle=nil;
@@ -390,13 +452,7 @@ name:(NSString *)name propertiesToAdd:(NSMutableDictionary *)props
 		parsersolidobj=nil;
 		firstsoliddict=prevsoliddict=nil;
 
-		// If the handle is a CSStreamHandle, it can not seek, so treat
-		// this like a solid archive (for instance, .tar.gz). Also, it will
-		// usually be wrapped in a CSSubHandle so unwrap it first.
-		CSHandle *testhandle=handle;
-		if([handle isKindOfClass:[CSSubHandle class]]) testhandle=[(CSSubHandle *)handle parentHandle];
-
-		if([testhandle isKindOfClass:[CSStreamHandle class]]) forcesolid=YES;
+		forcesolid=NO;
 
 		shouldstop=NO;
 	}
@@ -420,14 +476,61 @@ name:(NSString *)name propertiesToAdd:(NSMutableDictionary *)props
 
 
 
--(NSDictionary *)properties { return properties; }
+
+-(CSHandle *)handle { return sourcehandle; }
+
+-(void)setHandle:(CSHandle *)newhandle
+{
+	[sourcehandle autorelease];
+	sourcehandle=[newhandle retain];
+
+	// If the handle is a CSStreamHandle, it can not seek, so treat
+	// this like a solid archive (for instance, .tar.gz). Also, it will
+	// usually be wrapped in a CSSubHandle so unwrap it first.
+	CSHandle *testhandle=newhandle;
+	if([testhandle isKindOfClass:[CSSubHandle class]]) testhandle=[(CSSubHandle *)testhandle parentHandle];
+
+	if([testhandle isKindOfClass:[CSStreamHandle class]]) forcesolid=YES;
+	else forcesolid=NO;
+}
+
+-(XADResourceFork *)resourceFork { return resourcefork; }
+
+-(void)setResourceFork:(XADResourceFork *)newfork
+{
+	[resourcefork autorelease];
+	resourcefork=[newfork retain];
+}
 
 -(NSString *)name { return [properties objectForKey:XADArchiveNameKey]; }
 
+-(void)setName:(NSString *)newname
+{
+	[properties setObject:[newname lastPathComponent] forKey:XADArchiveNameKey];
+}
+
 -(NSString *)filename { return [[properties objectForKey:XADVolumesKey] objectAtIndex:0]; }
 
+-(void)setFilename:(NSString *)filename
+{
+	[properties setObject:[NSArray arrayWithObject:filename] forKey:XADVolumesKey];
+	[self setName:filename];
+}
+
 -(NSArray *)allFilenames { return [properties objectForKey:XADVolumesKey]; }
 
+-(void)setAllFilenames:(NSArray *)newnames
+{
+	[properties setObject:newnames forKey:XADVolumesKey];
+	[self setName:[newnames objectAtIndex:0]];
+}
+
+-(id)delegate { return delegate; }
+
+-(void)setDelegate:(id)newdelegate { delegate=newdelegate; }
+
+-(NSDictionary *)properties { return properties; }
+
 -(NSString *)currentFilename
 {
 	if([sourcehandle isKindOfClass:[XADMultiHandle class]])
@@ -446,13 +549,6 @@ name:(NSString *)name propertiesToAdd:(NSMutableDictionary *)props
 	return isencrypted&&[isencrypted boolValue];
 }
 
-
-
-
--(id)delegate { return delegate; }
-
--(void)setDelegate:(id)newdelegate { delegate=newdelegate; }
-
 -(NSString *)password
 {
 	if(!password)
@@ -660,6 +756,11 @@ static NSInteger XADVolumeSort(id entry1,id entry2,void *extptr)
 	else return [str1 compare:str2 options:NSCaseInsensitiveSearch];
 }
 
++(NSArray *)scanForVolumesWithFilename:(NSString *)filename regex:(XADRegex *)regex
+{
+	return [self scanForVolumesWithFilename:filename regex:regex firstFileExtension:nil];
+}
+
 +(NSArray *)scanForVolumesWithFilename:(NSString *)filename
 regex:(XADRegex *)regex firstFileExtension:(NSString *)firstext
 {
@@ -704,8 +805,6 @@ regex:(XADRegex *)regex firstFileExtension:(NSString *)firstext
 
 
 
--(CSHandle *)handle { return sourcehandle; }
-
 -(CSHandle *)handleAtDataOffsetForDictionary:(NSDictionary *)dict
 {
 	NSNumber *skipoffs=[dict objectForKey:XADSkipOffsetKey];
@@ -761,6 +860,7 @@ regex:(XADRegex *)regex firstFileExtension:(NSString *)firstext
 
 
 
+
 -(NSArray *)volumes
 {
 	if([sourcehandle respondsToSelector:@selector(handles)]) return [(id)sourcehandle handles];
@@ -878,8 +978,8 @@ regex:(XADRegex *)regex firstFileExtension:(NSString *)firstext
 		if(finderflags) [dict setObject:[NSNumber numberWithInt:finderflags] forKey:XADFinderFlagsKey];
 	}
 
-	// If this is an embedded archive that can't seek, force a solid flag.
-	if(forcesolid) [dict setObject:sourcehandle forKey:XADSolidObjectKey];
+	// If this is an embedded archive that can't seek, force a solid flag if one isn't already present.
+	if(forcesolid && ![dict objectForKey:XADSolidObjectKey]) [dict setObject:sourcehandle forKey:XADSolidObjectKey];
 
 	// Handle solidness - set FirstSolid, NextSolid and IsSolid depending on SolidObject.
 	id solidobj=[dict objectForKey:XADSolidObjectKey];
@@ -1072,6 +1172,12 @@ name:(NSString *)name propertiesToAdd:(NSMutableDictionary *)props
 	return [self recognizeFileWithHandle:handle firstBytes:data name:name];
 }
 
++(BOOL)recognizeFileWithHandle:(CSHandle *)handle firstBytes:(NSData *)data
+resourceFork:(XADResourceFork *)fork name:(NSString *)name propertiesToAdd:(NSMutableDictionary *)props
+{
+	return [self recognizeFileWithHandle:handle firstBytes:data name:name propertiesToAdd:props];
+}
+
 +(NSArray *)volumesForHandle:(CSHandle *)handle firstBytes:(NSData *)data
 name:(NSString *)name { return nil; }
 
