@@ -3,14 +3,15 @@
 
 @interface XADPDFParser:XADArchiveParser
 {
+	PDFParser *parser;
 }
 
--(id)initWithHandle:(CSHandle *)handle name:(NSString *)name;
--(void)dealloc;
-
 +(int)requiredHeaderSize;
 +(BOOL)recognizeFileWithHandle:(CSHandle *)handle firstBytes:(NSData *)data name:(NSString *)name;
 
+-(id)init;
+-(void)dealloc;
+
 -(void)parse;
 -(NSString *)compressionNameForStream:(PDFStream *)stream excludingLast:(BOOL)excludelast;
 
