@@ -66,6 +66,16 @@ static inline int imin(int a,int b) { return a<b?a:b; }
 		firstFileExtension:nil];
 	}
 
+	// In case the first part of a .1.zip multi-part file was detected as Zip,
+	// scan for the other parts.
+	if((matches=[name substringsCapturedByPattern:@"^(.*)\\.1\\.zip$" options:REG_ICASE]))
+	{
+		return [self scanForVolumesWithFilename:name
+		regex:[XADRegex regexWithPattern:[NSString stringWithFormat:@"^%@(\\.[0-9]+|())\\.zip$",
+			[[matches objectAtIndex:1] escapedPattern]] options:REG_ICASE]
+		firstFileExtension:nil];
+	}
+
 	return nil;
 }
 
@@ -96,7 +106,7 @@ static inline int imin(int a,int b) { return a<b?a:b; }
 	off_t end=[fh offsetInFile];
 
 	int numbytes=0x10011;
-	if(numbytes>end) numbytes=end;
+	if(numbytes>end) numbytes=(int)end;
 
 	uint8_t buf[numbytes];
 
@@ -165,7 +175,7 @@ static inline int imin(int a,int b) { return a<b?a:b; }
 	int centraldirstartdisk=[fh readUInt16LE];
 	/*off_t numentriesdisk=*/[fh readUInt16LE];
 	off_t numentries=[fh readUInt16LE];
-	/*off_t centralsize=*/[fh readUInt32LE];
+	off_t centralsize=[fh readUInt32LE];
 	off_t centraloffset=[fh readUInt32LE];
 	int commentlength=[fh readUInt16LE];
 
@@ -193,7 +203,7 @@ static inline int imin(int a,int b) { return a<b?a:b; }
 			centraldirstartdisk=[fh readUInt32LE];
 			/*off_t numentriesdisk=*/[fh readUInt64LE];
 			numentries=[fh readUInt64LE];
-			/*off_t centralsize=*/[fh readUInt64LE];
+			centralsize=[fh readUInt64LE];
 			centraloffset=[fh readUInt64LE];
 		}
 	}
@@ -263,6 +273,15 @@ static inline int imin(int a,int b) { return a<b?a:b; }
         
 		off_t next=[fh offsetInFile];
 
+		// Some idiotic compressors write files with more than 65535 files without
+		// using Zip64, so numentries overflows. Try to detect if there is enough space
+		// left in the central directory for another 65536 files, and if so, extend the
+		// parsing to include them. This may happen multiple times.
+		if(i==numentries-1 && zip64offs<0)
+		{
+			if(centraloffset+centralsize-next>65536*46) numentries+=65536;
+		}
+
 		// Read local header
 		[fh seekToFileOffset:[self offsetForVolume:startdisk offset:locheaderoffset]];
 
@@ -670,6 +689,16 @@ uncompressedSizePointer:(off_t *)uncompsizeptr compressedSizePointer:(off_t *)co
 				uint32_t crc=[fh readUInt32LE];
 				NSData *unicodedata=[fh readDataOfLength:size-5];
 
+				// Some archivers append garbage zero bytes to the end of the name.
+				// Remove them if necessary.
+				const uint8_t *bytes=[unicodedata bytes];
+				int length=size-5;
+				if(length && bytes[length-1]==0)
+				{
+					while(length && bytes[length-1]==0) length--;
+					unicodedata=[unicodedata subdataWithRange:NSMakeRange(0,length)];
+				}
+
 				if((XADCalculateCRC(0xffffffff,[namedata bytes],[namedata length],
 				XADCRCTable_edb88320)^0xffffffff)==crc)
 				{
