@@ -3,7 +3,7 @@
 #import "CSCommandLineParser.h"
 #import "CommandLineCommon.h"
 
-#define VERSION_STRING @"v1.7"
+#define VERSION_STRING @"v1.8"
 
 BOOL quietmode;
 
@@ -14,6 +14,10 @@ int numerrors;
 
 int main(int argc,const char **argv)
 {
+	#ifdef GNUSTEP
+	setlocale(LC_ALL,""); // GNUstep requires this.
+	#endif
+
 	NSAutoreleasePool *pool=[NSAutoreleasePool new];
 
 	CSCommandLineParser *cmdline=[[CSCommandLineParser new] autorelease];
